AbilityPlugin

Struct AbilityPlugin 

pub struct AbilityPlugin<A: Abilitylike> { /* private fields */ }
A Plugin that advances the abilities of type A.

This plugin ticks down the CooldownState and ChargeState components of every entity with abilities of type A, replenishing charges and refreshing cooldowns as they expire.

This plugin needs to be passed in an Abilitylike enum type that you’ve created for your game. If you have more than one distinct type of action (e.g. menu actions, camera actions and player actions), consider creating multiple Abilitylike enums and adding a copy of this plugin for each Abilitylike type.

Systems

The plugin adds tick_cooldowns to PreUpdate, in the AbilitySystem::TickCooldowns set. It is ordered before InputManagerSystem::Update, so cooldowns are advanced before action states are updated for the frame.

WARNING: These systems run during PreUpdate. If you have systems that care about inputs and actions that also run during this stage, you must define an ordering between your systems or behavior will be very erratic.