Struct leafwing_abilities::AbilityState
source · [−]pub struct AbilityState<A: Abilitylike> {
pub action_state: &'static ActionState<A>,
pub charges: &'static mut ChargeState<A>,
pub cooldowns: &'static mut CooldownState<A>,
}Expand description
A custom WorldQuery type that fetches all ability relevant data for you.
This type is intended to make collecting the data for Abilitylike methods easier when working with a full AbilitiesBundle.
This struct can be used as the first type parameter in a Query to fetch the appropriate data.
Once you have a AbilityStateItem by calling .iter_mut() or .single_mut on your query
(or a AbilityStateReadOnlyItem by calling .iter() or .single),
you can use the methods defined there to perform common tasks quickly and reliably.
Fields
action_state: &'static ActionState<A>The ActionState of the abilities of this entity of type A
charges: &'static mut ChargeState<A>The ChargeState associated with each action of type A for this entity
cooldowns: &'static mut CooldownState<A>The CooldownState associated with each action of type A for this entity